@@ -169,4 +169,10 @@ int _gnutls_selected_cert_supported_kx (struct gnutls_session_int *session,
 int
 _gnutls_check_key_cert_match (gnutls_certificate_credentials_t res);
 
+int _gnutls_gen_dhe_signature(gnutls_session_t session, gnutls_buffer_st* data,
+	uint8_t* plain, unsigned plain_size);
+int
+_gnutls_proc_dhe_signature (gnutls_session_t session, uint8_t * data,
+                    size_t _data_size, gnutls_datum_t *vparams);
+
 #endif
