@@ -7,7 +7,7 @@
  *
  * The GnuTLS is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
+ * as published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
@@ -68,6 +68,7 @@ _gnutls_set_psk_session_key (gnutls_session_t session,
 {
   gnutls_datum_t pwd_psk = { NULL, 0 };
   size_t dh_secret_size;
+  uint8_t * p;
   int ret;
 
   if (dh_secret == NULL)
@@ -77,9 +78,9 @@ _gnutls_set_psk_session_key (gnutls_session_t session,
 
   /* set the session key
    */
-  session->key->key.size = 4 + dh_secret_size + ppsk->size;
-  session->key->key.data = gnutls_malloc (session->key->key.size);
-  if (session->key->key.data == NULL)
+  session->key.key.size = 4 + dh_secret_size + ppsk->size;
+  session->key.key.data = gnutls_malloc (session->key.key.size);
+  if (session->key.key.data == NULL)
     {
       gnutls_assert ();
       ret = GNUTLS_E_MEMORY_ERROR;
@@ -92,12 +93,18 @@ _gnutls_set_psk_session_key (gnutls_session_t session,
    * (uint16_t) psk_size
    * the psk
    */
-  _gnutls_write_uint16 (dh_secret_size, session->key->key.data);
+  p = session->key.key.data;
+  _gnutls_write_uint16 (dh_secret_size, p);
+  p+=2;
   if (dh_secret == NULL)
-    memset (&session->key->key.data[2], 0, dh_secret_size);
+    memset (p, 0, dh_secret_size);
   else
-    memcpy (&session->key->key.data[2], dh_secret->data, dh_secret->size);
-  _gnutls_write_datum16 (&session->key->key.data[dh_secret_size + 2], *ppsk);
+    memcpy (p, dh_secret->data, dh_secret->size);
+
+  p += dh_secret_size;
+  _gnutls_write_uint16 (ppsk->size, p);
+  if (ppsk->data != NULL)
+    memcpy (p+2, ppsk->data, ppsk->size);
 
   ret = 0;
 
@@ -161,7 +168,7 @@ _gnutls_gen_psk_client_kx (gnutls_session_t session, gnutls_buffer_st* data)
   gnutls_psk_client_credentials_t cred;
 
   cred = (gnutls_psk_client_credentials_t)
-    _gnutls_get_cred (session->key, GNUTLS_CRD_PSK, NULL);
+    _gnutls_get_cred (session, GNUTLS_CRD_PSK, NULL);
 
   if (cred == NULL)
     {
@@ -210,7 +217,7 @@ _gnutls_proc_psk_client_kx (gnutls_session_t session, uint8_t * data,
   psk_auth_info_t info;
 
   cred = (gnutls_psk_server_credentials_t)
-    _gnutls_get_cred (session->key, GNUTLS_CRD_PSK, NULL);
+    _gnutls_get_cred (session, GNUTLS_CRD_PSK, NULL);
 
   if (cred == NULL)
     {
@@ -285,7 +292,7 @@ _gnutls_gen_psk_server_kx (gnutls_session_t session, gnutls_buffer_st* data)
   gnutls_datum_t hint;
 
   cred = (gnutls_psk_server_credentials_t)
-    _gnutls_get_cred (session->key, GNUTLS_CRD_PSK, NULL);
+    _gnutls_get_cred (session, GNUTLS_CRD_PSK, NULL);
 
   if (cred == NULL)
     {
@@ -320,7 +327,7 @@ _gnutls_proc_psk_server_kx (gnutls_session_t session, uint8_t * data,
   psk_auth_info_t info;
 
   cred = (gnutls_psk_client_credentials_t)
-    _gnutls_get_cred (session->key, GNUTLS_CRD_PSK, NULL);
+    _gnutls_get_cred (session, GNUTLS_CRD_PSK, NULL);
 
   if (cred == NULL)
     {
