@@ -293,10 +293,14 @@ static const int protocol_priority_suiteb[] = {
 
 static const int kx_priority_performance[] = {
   GNUTLS_KX_RSA,
+#ifdef ENABLE_ECDHE
   GNUTLS_KX_ECDHE_ECDSA,
   GNUTLS_KX_ECDHE_RSA,
+#endif
+#ifdef ENABLE_DHE
   GNUTLS_KX_DHE_RSA,
   GNUTLS_KX_DHE_DSS,
+#endif
   0
 };
 
@@ -307,11 +311,17 @@ static const int kx_priority_suiteb[] = {
 
 static const int kx_priority_export[] = {
   GNUTLS_KX_RSA,
+#ifdef ENABLE_ECDHE
   GNUTLS_KX_ECDHE_ECDSA,
   GNUTLS_KX_ECDHE_RSA,
+#endif
+#ifdef ENABLE_DHE
   GNUTLS_KX_DHE_RSA,
   GNUTLS_KX_DHE_DSS,
+#endif
+#ifdef ENABLE_RSA_EXPORT
   GNUTLS_KX_RSA_EXPORT,
+#endif
   0
 };
 
@@ -319,11 +329,19 @@ static const int kx_priority_secure[] = {
   /* The ciphersuites that offer forward secrecy take
    * precedence
    */
+#ifdef ENABLE_ECDHE
   GNUTLS_KX_ECDHE_ECDSA,
   GNUTLS_KX_ECDHE_RSA,
+#endif
+  GNUTLS_KX_RSA,
+  /* KX-RSA is now ahead of DHE-RSA and DHE-DSS due to the compatibility
+   * issues the DHE ciphersuites have. That is, one cannot enforce a specific
+   * security level without dropping the connection. 
+   */
+#ifdef ENABLE_DHE
   GNUTLS_KX_DHE_RSA,
   GNUTLS_KX_DHE_DSS,
-  GNUTLS_KX_RSA,
+#endif
   /* GNUTLS_KX_ANON_DH: Man-in-the-middle prone, don't add!
    * GNUTLS_KX_RSA_EXPORT: Deprecated, don't add!
    */
@@ -691,9 +709,9 @@ bulk_rmadd_func *func;
                      sign_priority_secure128);
       func (&priority_cache->supported_ecc, supported_ecc_secure128);
 
-      /* be conservative for now. Set the bits to correspond to an 80-bit level */
+      /* be conservative for now. Set the bits to correspond to an 72-bit level */
       if (priority_cache->level == 0)
-        priority_cache->level = GNUTLS_SEC_PARAM_LOW;
+        priority_cache->level = GNUTLS_SEC_PARAM_WEAK;
       return 1;
     }
   else if (strcasecmp (level, LEVEL_SUITEB128) == 0)
