@@ -7,7 +7,7 @@
  *
  * The GnuTLS is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
+ * as published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
@@ -417,6 +417,9 @@ cleanup:
  * #gnutls_privkey_t structure. At least one of the two callbacks
  * must be non-null.
  *
+ * Note that the signing function is supposed to "raw" sign data, i.e.,
+ * without any hashing or preprocessing.
+ *
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
  *   negative error value.
  *
@@ -449,6 +452,9 @@ gnutls_privkey_import_ext (gnutls_privkey_t pkey,
  * must be non-null. If a deinitialization function is provided
  * then flags is assumed to contain %GNUTLS_PRIVKEY_IMPORT_AUTO_RELEASE.
  *
+ * Note that the signing function is supposed to "raw" sign data, i.e.,
+ * without any hashing or preprocessing.
+ *
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
  *   negative error value.
  *
@@ -728,7 +734,7 @@ gnutls_privkey_sign_data (gnutls_privkey_t signer,
       goto cleanup;
     }
 
-  ret = _gnutls_privkey_sign_hash (signer, &digest, signature);
+  ret = gnutls_privkey_sign_raw_data (signer, flags, &digest, signature);
   _gnutls_free_datum (&digest);
 
   if (ret < 0)
@@ -791,7 +797,7 @@ gnutls_privkey_sign_hash (gnutls_privkey_t signer,
       goto cleanup;
     }
 
-  ret = _gnutls_privkey_sign_hash (signer, &digest, signature);
+  ret = gnutls_privkey_sign_raw_data (signer, flags, &digest, signature);
   if (ret < 0)
     {
       gnutls_assert ();
@@ -805,42 +811,49 @@ cleanup:
   return ret;
 }
 
-/*-
- * _gnutls_privkey_sign_hash:
+/**
+ * gnutls_privkey_sign_raw_data:
  * @key: Holds the key
+ * @flags: should be zero
  * @data: holds the data to be signed
  * @signature: will contain the signature allocate with gnutls_malloc()
  *
  * This function will sign the given data using a signature algorithm
- * supported by the private key.
+ * supported by the private key. Note that this is a low-level function
+ * and does not apply any preprocessing or hash on the signed data. 
+ * For example on an RSA key the input @data should be of the DigestInfo
+ * PKCS #1 1.5 format. Use it only if you know what are you doing.
  *
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
  * negative error value.
- -*/
+ *
+ * Since: 3.1.10
+ **/
 int
-_gnutls_privkey_sign_hash (gnutls_privkey_t key,
-                           const gnutls_datum_t * hash,
-                           gnutls_datum_t * signature)
+gnutls_privkey_sign_raw_data (gnutls_privkey_t key,
+				unsigned flags,
+                                const gnutls_datum_t * data,
+                                gnutls_datum_t * signature)
 {
   switch (key->type)
     {
 #ifdef ENABLE_OPENPGP
     case GNUTLS_PRIVKEY_OPENPGP:
       return gnutls_openpgp_privkey_sign_hash (key->key.openpgp,
-                                                hash, signature);
+                                                data, signature);
 #endif
 #ifdef ENABLE_PKCS11
     case GNUTLS_PRIVKEY_PKCS11:
       return _gnutls_pkcs11_privkey_sign_hash (key->key.pkcs11,
-                                               hash, signature);
+                                               data, signature);
 #endif
     case GNUTLS_PRIVKEY_X509:
       return _gnutls_pk_sign (key->key.x509->pk_algorithm,
-                              signature, hash, &key->key.x509->params);
+                              signature, data, &key->key.x509->params);
     case GNUTLS_PRIVKEY_EXT:
       if (key->key.ext.sign_func == NULL)
         return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
-      return key->key.ext.sign_func(key, key->key.ext.userdata, hash, signature);
+      return key->key.ext.sign_func(key, key->key.ext.userdata, data, signature);
     default:
       gnutls_assert ();
       return GNUTLS_E_INVALID_REQUEST;
@@ -966,14 +979,20 @@ cleanup:
 int
 gnutls_privkey_import_url (gnutls_privkey_t key, const char *url, unsigned int flags)
 {
+  if (strncmp(url, "pkcs11:", 7) == 0)
 #ifdef ENABLE_PKCS11
-  if (strstr(url, "pkcs11:") != NULL)
     return gnutls_privkey_import_pkcs11_url(key, url);
+#else
+    return gnutls_assert_val(GNUTLS_E_UNIMPLEMENTED_FEATURE);
 #endif
+
+  if (strncmp(url, "tpmkey:", 7) == 0)
 #ifdef HAVE_TROUSERS
-  if (strstr(url, "tpmkey:") != NULL)
     return gnutls_privkey_import_tpm_url(key, url, NULL, NULL, 0);
+#else
+    return gnutls_assert_val(GNUTLS_E_UNIMPLEMENTED_FEATURE);
 #endif
+
   return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
 }
 
@@ -999,3 +1018,32 @@ void gnutls_privkey_set_pin_function (gnutls_privkey_t key,
   key->pin.cb = fn;
   key->pin.data = userdata;
 }
+
+/**
+ * gnutls_privkey_status:
+ * @key: Holds the key
+ *
+ * Checks the status of the private key token. This function
+ * is an actual wrapper over gnutls_pkcs11_privkey_status(), and
+ * if the private key is a PKCS #11 token it will check whether
+ * it is inserted or not.
+ *
+ * Returns: this function will return non-zero if the token 
+ * holding the private key is still available (inserted), and zero otherwise.
+ * 
+ * Since: 3.1.10
+ *
+ **/
+int
+gnutls_privkey_status (gnutls_privkey_t key)
+{
+  switch (key->type)
+    {
+#ifdef ENABLE_PKCS11
+    case GNUTLS_PRIVKEY_PKCS11:
+      return gnutls_pkcs11_privkey_status (key->key.pkcs11);
+#endif
+    default:
+      return 1;
+    }
+}
