@@ -87,7 +87,7 @@ check_bits (gnutls_session_t session, gnutls_x509_crt_t crt, unsigned int max_bi
 
 /* three days */
 #define MAX_OCSP_VALIDITY_SECS (3*60*60*24)
-
+#ifdef ENABLE_OCSP
 /* If the certificate is revoked status will be GNUTLS_CERT_REVOKED.
  * 
  * Returns:
@@ -198,6 +198,7 @@ cleanup:
   return ret;
 }
 
+#endif
 
 #define CLEAR_CERTS for(x=0;x<peer_certificate_list_size;x++) { \
 	if (peer_certificate_list[x]) \
@@ -321,12 +322,14 @@ _gnutls_x509_cert_verify_peers (gnutls_session_t session,
         }
     }
 
+#ifdef ENABLE_OCSP
   ret = check_ocsp_response(session, peer_certificate_list[0], issuer, &resp, &ocsp_status);
   if (ret < 0)
     {
       CLEAR_CERTS;
       return gnutls_assert_val(ret);
     }
+#endif
 
 skip_ocsp:
   /* Verify certificate 
@@ -867,6 +870,9 @@ cleanup:
   gnutls_free (ccert);
   return ret;
 }
+#else
+# define read_cert_url(x,y) gnutls_assert_val(GNUTLS_E_UNIMPLEMENTED_FEATURE)
+# define read_cas_url(x,y) gnutls_assert_val(GNUTLS_E_UNIMPLEMENTED_FEATURE)
 #endif
 
 /* Reads a certificate file
@@ -879,12 +885,10 @@ read_cert_file (gnutls_certificate_credentials_t res,
   size_t size;
   char *data;
 
-#ifdef ENABLE_PKCS11
   if (strncmp (certfile, "pkcs11:", 7) == 0)
     {
       return read_cert_url (res, certfile);
     }
-#endif /* ENABLE_PKCS11 */
 
   data = read_binary_file (certfile, &size);
 
@@ -914,10 +918,13 @@ read_key_file (gnutls_certificate_credentials_t res,
   size_t size;
   char *data;
 
-  if (gnutls_url_is_supported(keyfile))
+  if (_gnutls_url_is_known(keyfile)) 
     {
-      return read_key_url (res, keyfile);
-    }
+      if (gnutls_url_is_supported(keyfile))
+        return read_key_url (res, keyfile);
+      else
+        return gnutls_assert_val(GNUTLS_E_UNIMPLEMENTED_FEATURE);
+  }
 
   data = read_binary_file (keyfile, &size);
 
@@ -1663,12 +1670,10 @@ gnutls_certificate_set_x509_trust_file (gnutls_certificate_credentials_t cred,
   gnutls_datum_t cas;
   size_t size;
 
-#ifdef ENABLE_PKCS11
   if (strncmp (cafile, "pkcs11:", 7) == 0)
     {
       return read_cas_url (cred, cafile);
     }
-#endif
 
   cas.data = (void*)read_binary_file (cafile, &size);
   if (cas.data == NULL)
@@ -2148,3 +2153,41 @@ void gnutls_certificate_set_pin_function (gnutls_certificate_credentials_t cred,
   cred->pin.cb = fn;
   cred->pin.data = userdata;
 }
+
+/**
+ * gnutls_url_is_supported:
+ * @url: A PKCS 11 url
+ *
+ * Check whether url is supported.  Depending on the system libraries
+ * GnuTLS may support pkcs11 or tpmkey URLs.
+ *
+ * Returns: return non-zero if the given URL is supported, and zero if
+ * it is not known.
+ *
+ * Since: 3.1.0
+ **/
+int
+gnutls_url_is_supported (const char* url)
+{
+#ifdef ENABLE_PKCS11
+  if (strstr(url, "pkcs11:") != NULL)
+    return 1;
+#endif
+#ifdef HAVE_TROUSERS
+  if (strstr(url, "tpmkey:") != NULL)
+    return 1;
+#endif
+  return 0;
+}
+
+int
+_gnutls_url_is_known (const char* url)
+{
+  if (strstr(url, "pkcs11:") != NULL)
+    return 1;
+  else if (strstr(url, "tpmkey:") != NULL)
+    return 1;
+  else
+    return 0;
+}
+
