@@ -7,7 +7,7 @@
  *
  * The GnuTLS is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
+ * as published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
@@ -31,6 +31,7 @@
 #include <x509_b64.h>
 #include "x509_int.h"
 #include <common.h>
+#include <c-ctype.h>
 
 struct oid_to_string
 {
@@ -47,15 +48,14 @@ struct oid_to_string
 static const struct oid_to_string _oid2str[] = {
   /* PKIX
    */
-  {"1.3.6.1.5.5.7.9.1", "dateOfBirth", NULL, ASN1_ETYPE_GENERALIZED_TIME},
   {"1.3.6.1.5.5.7.9.2", "placeOfBirth", "PKIX1.DirectoryString", ASN1_ETYPE_INVALID},
   {"1.3.6.1.5.5.7.9.3", "gender", NULL, ASN1_ETYPE_PRINTABLE_STRING},
   {"1.3.6.1.5.5.7.9.4", "countryOfCitizenship", NULL, ASN1_ETYPE_PRINTABLE_STRING},
   {"1.3.6.1.5.5.7.9.5", "countryOfResidence", NULL, ASN1_ETYPE_PRINTABLE_STRING},
 
   {"2.5.4.6", "C", NULL, ASN1_ETYPE_PRINTABLE_STRING},
-  {"2.5.4.9", "STREET", "PKIX1.DirectoryString", ASN1_ETYPE_INVALID},
-  {"2.5.4.12", "T", "PKIX1.DirectoryString", ASN1_ETYPE_INVALID},
+  {"2.5.4.9", "street", "PKIX1.DirectoryString", ASN1_ETYPE_INVALID},
+  {"2.5.4.12", "title", "PKIX1.DirectoryString", ASN1_ETYPE_INVALID},
   {"2.5.4.10", "O", "PKIX1.DirectoryString", ASN1_ETYPE_INVALID},
   {"2.5.4.11", "OU", "PKIX1.DirectoryString", ASN1_ETYPE_INVALID},
   {"2.5.4.3", "CN", "PKIX1.DirectoryString", ASN1_ETYPE_INVALID},
@@ -71,7 +71,7 @@ static const struct oid_to_string _oid2str[] = {
   {"2.5.4.65", "pseudonym", "PKIX1.DirectoryString", ASN1_ETYPE_INVALID},
   {"2.5.4.46", "dnQualifier", NULL, ASN1_ETYPE_PRINTABLE_STRING},
   {"2.5.4.17", "postalCode", "PKIX1.DirectoryString", ASN1_ETYPE_INVALID},
-  {"2.5.4.41", "Name", "PKIX1.DirectoryString", ASN1_ETYPE_INVALID},
+  {"2.5.4.41", "name", "PKIX1.DirectoryString", ASN1_ETYPE_INVALID},
   {"2.5.4.15", "businessCategory", "PKIX1.DirectoryString", ASN1_ETYPE_INVALID},
 
   {"0.9.2342.19200300.100.1.25", "DC", NULL, ASN1_ETYPE_IA5_STRING},
@@ -116,9 +116,26 @@ static const struct oid_to_string* get_oid_entry (const char* oid)
   while (_oid2str[i].oid != NULL);
 
   return NULL;
-
 }
 
+const char* _gnutls_ldap_string_to_oid (const char* str, unsigned str_len)
+{
+  unsigned int i = 0;
+
+  do
+    {
+      if ((_oid2str[i].ldap_desc != NULL) &&
+          (str_len == strlen(_oid2str[i].ldap_desc)) && 
+          (strncasecmp (_oid2str[i].ldap_desc, str, str_len) == 0))
+        return _oid2str[i].oid;
+      i++;
+    }
+  while (_oid2str[i].oid != NULL);
+
+  return NULL;
+}
+
+
 /**
  * gnutls_x509_dn_oid_known:
  * @oid: holds an Object Identifier in a null terminated string
@@ -206,7 +223,7 @@ size_t size;
        * characters then treat it as printable.
        */
       for (i = 0; i < input->size; i++)
-        if (!isascii (input->data[i]))
+        if (!c_isascii (input->data[i]))
           ascii = 1;
 
       if (ascii == 0)
@@ -1782,3 +1799,35 @@ cleanup:
   
   return ret;                          
 }
+
+int
+_gnutls_x509_get_raw_dn2 (ASN1_TYPE c2, gnutls_datum_t* raw,
+                              const char *whom, gnutls_datum_t * dn)
+{
+  int result, len1;
+  int start1, end1;
+  result =
+    asn1_der_decoding_startEnd (c2, raw->data, raw->size,
+                                whom, &start1, &end1);
+
+  if (result != ASN1_SUCCESS)
+    {
+      gnutls_assert ();
+      result = _gnutls_asn2err (result);
+      goto cleanup;
+    }
+
+  len1 = end1 - start1 + 1;
+
+  result = _gnutls_set_datum (dn, &raw->data[start1], len1);
+  if (result < 0)
+    {
+      gnutls_assert();
+      goto cleanup;
+    }
+
+  result = 0;
+
+cleanup:
+  return result;
+}
