@@ -1,10 +1,9 @@
 /* gc-pbkdf2-sha1.c --- Password-Based Key Derivation Function a'la PKCS#5
-   Copyright (C) 2002-2006, 2008, 2010, 2012 Free Software Foundation,
-   Inc.
+   Copyright (C) 2002-2012 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
-   the Free Software Foundation; either version 3, or (at your option)
+   the Free Software Foundation; either version 2.1, or (at your option)
    any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -54,8 +53,8 @@
 
 int
 _gnutls_pbkdf2_sha1 (const char *P, size_t Plen,
-                     const char *S, size_t Slen,
-                     unsigned int c, char *DK, size_t dkLen)
+                     const unsigned char *S, size_t Slen,
+                     unsigned int c, unsigned char *DK, size_t dkLen)
 {
   unsigned int hLen = 20;
   char U[20];
