@@ -7,7 +7,7 @@
  *
  * The GnuTLS is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
+ * as published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
@@ -70,6 +70,7 @@ gnutls_x509_privkey_deinit (gnutls_x509_privkey_t key)
   if (!key)
     return;
 
+  gnutls_pk_params_clear(&key->params);
   gnutls_pk_params_release(&key->params);
   asn1_delete_structure (&key->key);
   gnutls_free (key);
@@ -224,6 +225,7 @@ _gnutls_privkey_decode_pkcs1_rsa_key (const gnutls_datum_t * raw_key,
 
 error:
   asn1_delete_structure (&pkey_asn);
+  gnutls_pk_params_clear (&pkey->params);
   gnutls_pk_params_release (&pkey->params);
   return NULL;
 
@@ -300,7 +302,7 @@ _gnutls_privkey_decode_ecc_key (const gnutls_datum_t * raw_key,
     }
 
   /* read the public key */
-  ret = _gnutls_x509_read_value(pkey_asn, "publicKey", &out, 2);
+  ret = _gnutls_x509_read_value (pkey_asn, "publicKey", &out);
   if (ret < 0)
     {
       gnutls_assert();
@@ -331,6 +333,7 @@ _gnutls_privkey_decode_ecc_key (const gnutls_datum_t * raw_key,
 
 error:
   asn1_delete_structure (&pkey_asn);
+  gnutls_pk_params_clear (&pkey->params);
   gnutls_pk_params_release (&pkey->params);
   return NULL;
 
@@ -401,6 +404,7 @@ decode_dsa_key (const gnutls_datum_t * raw_key, gnutls_x509_privkey_t pkey)
 
 error:
   asn1_delete_structure (&dsa_asn);
+  gnutls_pk_params_clear(&pkey->params);
   gnutls_pk_params_release(&pkey->params);
   return NULL;
 
@@ -558,10 +562,53 @@ failover:
   return result;
 }
 
+static int import_pkcs12_privkey (gnutls_x509_privkey_t key,
+                                  const gnutls_datum_t * data,
+                                  gnutls_x509_crt_fmt_t format,
+                                  const char* password, unsigned int flags)
+{
+int ret;
+gnutls_pkcs12_t p12;
+gnutls_x509_privkey_t newkey;
+
+  ret = gnutls_pkcs12_init(&p12);
+  if (ret < 0)
+    return gnutls_assert_val(ret);
+  
+  ret = gnutls_pkcs12_import(p12, data, format, flags);
+  if (ret < 0)
+    {
+      gnutls_assert();
+      goto fail;
+    }
+
+  ret = gnutls_pkcs12_simple_parse (p12, password, &newkey, NULL, NULL, NULL, NULL, NULL, 0);
+  if (ret < 0)
+    {
+      gnutls_assert();
+      goto fail;
+    }
+
+  ret = gnutls_x509_privkey_cpy (key, newkey);
+  gnutls_x509_privkey_deinit (newkey);
+  if (ret < 0)
+    {
+      gnutls_assert();
+      goto fail;
+    }
+  
+  ret = 0;
+fail:
+  
+  gnutls_pkcs12_deinit(p12);
+  
+  return ret;
+}
+
 /**
  * gnutls_x509_privkey_import2:
  * @key: The structure to store the parsed key
- * @data: The DER or PEM encoded certificate.
+ * @data: The DER or PEM encoded key.
  * @format: One of DER or PEM
  * @password: A password (optional)
  * @flags: an ORed sequence of gnutls_pkcs_encrypt_flags_t
@@ -570,8 +617,8 @@ failover:
  * the native #gnutls_x509_privkey_t format, irrespective of the
  * input format. The input format is auto-detected.
  *
- * The supported formats are typical X.509, PKCS #8 and the openssl
- * format.
+ * The supported formats are basic unencrypted key, PKCS #8, PKCS #12,
+ * and the openssl format.
  *
  * If the provided key is encrypted but no password was given, then
  * %GNUTLS_E_DECRYPTION_FAILED is returned.
@@ -601,13 +648,15 @@ gnutls_x509_privkey_import2 (gnutls_x509_privkey_t key,
       ret = gnutls_x509_privkey_import_pkcs8(key, data, format, password, flags);
       if (ret < 0)
         {
-          if (format == GNUTLS_X509_FMT_PEM)
+          if (ret == GNUTLS_E_DECRYPTION_FAILED) goto cleanup;
+          ret = import_pkcs12_privkey(key, data, format, password, flags);
+          if (ret < 0 && format == GNUTLS_X509_FMT_PEM)
             {
-              int err;
-              err = gnutls_x509_privkey_import_openssl(key, data, password);
-              if (err < 0)
+	      if (ret == GNUTLS_E_DECRYPTION_FAILED) goto cleanup;
+
+              ret = gnutls_x509_privkey_import_openssl(key, data, password);
+              if (ret < 0)
                 {
-                  if (err == GNUTLS_E_DECRYPTION_FAILED) ret = err;
                   gnutls_assert();
                   goto cleanup;
                 }
@@ -793,6 +842,7 @@ gnutls_x509_privkey_import_rsa_raw2 (gnutls_x509_privkey_t key,
   return 0;
 
 cleanup:
+  gnutls_pk_params_clear(&key->params);
   gnutls_pk_params_release(&key->params);
   return ret;
 
@@ -884,6 +934,7 @@ gnutls_x509_privkey_import_dsa_raw (gnutls_x509_privkey_t key,
   return 0;
 
 cleanup:
+  gnutls_pk_params_clear(&key->params);
   gnutls_pk_params_release(&key->params);
   return ret;
 
@@ -956,6 +1007,7 @@ gnutls_x509_privkey_import_ecc_raw (gnutls_x509_privkey_t key,
   return 0;
 
 cleanup:
+  gnutls_pk_params_clear(&key->params);
   gnutls_pk_params_release(&key->params);
   return ret;
 
@@ -1348,6 +1400,7 @@ gnutls_x509_privkey_export_rsa_raw2 (gnutls_x509_privkey_t key,
         }
     }
 
+  gnutls_pk_params_clear(&pk_params);
   gnutls_pk_params_release (&pk_params);
 
   return 0;
@@ -1358,6 +1411,7 @@ error:
   _gnutls_free_datum (e);
   _gnutls_free_datum (p);
   _gnutls_free_datum (q);
+  gnutls_pk_params_clear(&pk_params);
   gnutls_pk_params_release (&pk_params);
 
   return ret;
@@ -1500,6 +1554,7 @@ gnutls_x509_privkey_generate (gnutls_x509_privkey_t key,
 
 cleanup:
   key->pk_algorithm = GNUTLS_PK_UNKNOWN;
+  gnutls_pk_params_clear(&key->params);
   gnutls_pk_params_release(&key->params);
 
   return ret;
