@@ -204,6 +204,25 @@ void dane_state_deinit(dane_state_t s)
 	free(s);
 }
 
+/**
+ * dane_state_set_dlv_file:
+ * @s: The structure to be deinitialized
+ * @file: The file holding the DLV keys.
+ *
+ * This function will set a file with trusted keys 
+ * for DLV  (DNSSEC  Lookaside  Validation).
+ *
+ **/
+int dane_state_set_dlv_file(dane_state_t s, const char* file)
+{
+int ret;
+  
+  ret = ub_ctx_set_option(s->ctx, (char*)"dlv-anchor-file:", (void*)file);
+  if (ret != 0)
+    return gnutls_assert_val(DANE_E_FILE_ERROR);
+  
+  return 0;
+}
 
 /**
  * dane_query_deinit:
@@ -274,7 +293,7 @@ int dane_query_tlsa(dane_state_t s, dane_query_t *r, const char* host, const cha
 	
 	(*r)->data_entries = i;
 
-	if (!(*r)->result->secure) {
+	if (!(s->flags & DANE_F_INSECURE) && !(*r)->result->secure) {
 		if ((*r)->result->bogus)
 			ret = gnutls_assert_val(DANE_E_INVALID_DNSSEC_SIG);
 		else
@@ -401,6 +420,8 @@ static int verify_ca(const gnutls_datum_t *raw_crt, unsigned raw_crt_size,
 {
 gnutls_datum_t pubkey = {NULL, 0};
 int ret;
+unsigned int vstatus;
+gnutls_x509_crt_t crt = NULL, ca = NULL;
 
 	if (raw_crt_size < 2)
 		return gnutls_assert_val(DANE_E_INVALID_REQUEST);
@@ -423,11 +444,57 @@ int ret;
                         gnutls_assert();
 			*verify |= DANE_VERIFY_CA_CONSTRAINS_VIOLATED;
                 }
+	} else {
+		ret = gnutls_assert_val(DANE_E_UNKNOWN_DANE_DATA);
+		goto cleanup;
 	}
+	
+	/* check if the certificate chain is actually a chain */
+	ret = gnutls_x509_crt_init(&crt);
+  	if (ret < 0) {
+  	  	ret = gnutls_assert_val(DANE_E_CERT_ERROR);
+  	  	goto cleanup;
+	}
+
+	ret = gnutls_x509_crt_init(&ca);
+  	if (ret < 0) {
+  	  	ret = gnutls_assert_val(DANE_E_CERT_ERROR);
+  	  	goto cleanup;
+	}
+
+	ret = gnutls_x509_crt_import(crt, &raw_crt[0], GNUTLS_X509_FMT_DER);
+  	if (ret < 0) {
+  	  	ret = gnutls_assert_val(DANE_E_CERT_ERROR);
+  	  	goto cleanup;
+	}
+
+	ret = gnutls_x509_crt_import(ca, &raw_crt[1], GNUTLS_X509_FMT_DER);
+  	if (ret < 0) {
+  	  	ret = gnutls_assert_val(DANE_E_CERT_ERROR);
+  	  	goto cleanup;
+	}
+	    
+	ret = gnutls_x509_crt_check_issuer(crt, ca);
+	if (ret == 0) {
+		gnutls_assert();
+		*verify |= DANE_VERIFY_CA_CONSTRAINS_VIOLATED;
+	}
+
+	ret = gnutls_x509_crt_verify(crt, &ca, 1, 0, &vstatus);
+	if (ret < 0) {
+  	  	ret = gnutls_assert_val(DANE_E_CERT_ERROR);
+  	  	goto cleanup;
+	}
+	if (vstatus != 0)
+		*verify |= DANE_VERIFY_CA_CONSTRAINS_VIOLATED;
 
 	ret = 0;
 cleanup:
 	free(pubkey.data);
+	if (crt != NULL)
+	  gnutls_x509_crt_deinit(crt);
+	if (ca != NULL)
+	  gnutls_x509_crt_deinit(ca);
 	return ret;
 }
 
@@ -457,6 +524,9 @@ int ret;
 		        gnutls_assert();
 			*verify |= DANE_VERIFY_CERT_DIFFERS;
                 }
+	} else {
+		ret = gnutls_assert_val(DANE_E_UNKNOWN_DANE_DATA);
+		goto cleanup;
 	}
 
 	ret = 0;
@@ -475,7 +545,7 @@ cleanup:
  * @proto: The protocol of the service connecting (e.g. tcp)
  * @port: The port of the service connecting (e.g. 443)
  * @sflags: Flags for the the initialization of @s (if NULL)
- * @vflags: Verification flags; should be zero
+ * @vflags: Verification flags; an OR'ed list of %dane_verify_flags_t.
  * @verify: An OR'ed list of %dane_verify_status_t.
  *
  * This function will verify the given certificate chain against the
@@ -483,6 +553,9 @@ cleanup:
  * If no information via DANE can be obtained the flag %DANE_VERIFY_NO_DANE_INFO
  * is set. If a DNSSEC signature is not available for the DANE 
  * record then the verify flag %DANE_VERIFY_NO_DNSSEC_DATA is set.
+ *
+ * Note that the CA constraint only applies for the directly certifying CA
+ * and does not account for long CA chains.
  * 
  * Due to the many possible options of DANE, there is no single threat
  * model countered. When notifying the user about DANE verification results
@@ -505,6 +578,7 @@ int dane_verify_crt (dane_state_t s,
 dane_state_t _s = NULL;
 dane_query_t r = NULL;
 int ret;
+unsigned checked = 0;
 unsigned int usage, type, match, idx;
 gnutls_datum_t data;
 	
@@ -538,24 +612,28 @@ gnutls_datum_t data;
 			gnutls_assert();
 			goto cleanup;
 		}
-	
-		if (usage == DANE_CERT_USAGE_LOCAL_CA || usage == DANE_CERT_USAGE_CA) {
+		
+		if (!(vflags & DANE_VFLAG_ONLY_CHECK_EE_USAGE) && (usage == DANE_CERT_USAGE_LOCAL_CA || usage == DANE_CERT_USAGE_CA)) {
 			ret = verify_ca(chain, chain_size, chain_type, type, match, &data, verify);
 			if (ret < 0) {
 				gnutls_assert();
 				goto cleanup;
                         }
-		
-		} else if (usage == DANE_CERT_USAGE_LOCAL_EE || usage == DANE_CERT_USAGE_EE) {
+                        checked = 1;
+		} else if (!(vflags & DANE_VFLAG_ONLY_CHECK_CA_USAGE) && (usage == DANE_CERT_USAGE_LOCAL_EE || usage == DANE_CERT_USAGE_EE)) {
 			ret = verify_ee(&chain[0], chain_type, type, match, &data, verify);
 			if (ret < 0) {
 				gnutls_assert();
 				goto cleanup;
                         }
+                        checked = 1;
 		}
 	} while(1);
 
-	ret = 0;
+	if ((vflags & DANE_VFLAG_FAIL_IF_NOT_CHECKED) && checked == 0)
+		ret = gnutls_assert_val(DANE_E_REQUESTED_DATA_NOT_AVAILABLE);
+	else
+		ret = 0;
 
 cleanup:
 	if (s == NULL) dane_state_deinit(_s);
