@@ -26,6 +26,7 @@
 #include <gnutls/pkcs12.h>
 #include <gnutls/pkcs11.h>
 #include <gnutls/abstract.h>
+#include <gnutls/crypto.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -78,6 +79,7 @@ FILE *infile;
 static gnutls_digest_algorithm_t default_dig;
 static unsigned int incert_format, outcert_format;
 static unsigned int req_key_type;
+gnutls_certificate_print_formats_t full_format = GNUTLS_CRT_PRINT_FULL;
 
 /* non interactive operation if set
  */
@@ -112,14 +114,14 @@ generate_private_key_int (common_info_st * cinfo)
   if (ret < 0)
     error (EXIT_FAILURE, 0, "privkey_init: %s", gnutls_strerror (ret));
 
-  bits = get_bits (key_type, cinfo->bits, cinfo->sec_param);
+  bits = get_bits (key_type, cinfo->bits, cinfo->sec_param, 1);
 
   fprintf (stderr, "Generating a %d bit %s private key...\n",
            bits, gnutls_pk_algorithm_get_name (key_type));
 
   if (bits > 1024 && key_type == GNUTLS_PK_DSA)
     fprintf (stderr,
-             "Note that DSA keys with size over 1024 can only be used with TLS 1.2 or later.\n\n");
+             "Note that DSA keys with size over 1024 may cause incompatibility problems when used with earlier than TLS 1.2 versions.\n\n");
 
   ret = gnutls_x509_privkey_generate (key, key_type, bits, 0);
   if (ret < 0)
@@ -137,7 +139,7 @@ cipher_to_flags (const char *cipher)
 {
   if (cipher == NULL)
     {
-      return GNUTLS_PKCS_USE_PBES2_AES_128;
+      return GNUTLS_PKCS_USE_PKCS12_ARCFOUR;
     }
   else if (strcasecmp (cipher, "3des") == 0)
     {
@@ -172,28 +174,6 @@ cipher_to_flags (const char *cipher)
   return -1;
 }
 
-static const char* get_password(common_info_st * cinfo, unsigned int *flags, int confirm)
-{
-  if (cinfo->null_password)
-    {
-      if (flags) *flags |= GNUTLS_PKCS_NULL_PASSWORD;
-      return NULL;
-    }
-  else if (cinfo->password)
-    {
-      if (cinfo->password[0] == 0 && flags)
-        *flags |= GNUTLS_PKCS_PLAIN;
-      return cinfo->password;
-    }
-  else
-    {
-      if (confirm)
-        return get_confirmed_pass (true);
-      else
-        return get_pass ();
-    }
-}
-
 
 static void
 print_private_key (common_info_st* cinfo, gnutls_x509_privkey_t key)
@@ -291,18 +271,22 @@ generate_certificate (gnutls_privkey_t * ret_key,
             error (EXIT_FAILURE, 0, "set_proxy_dn: %s",
                    gnutls_strerror (result));
 
+          get_dn_crt_set (crt);
           get_cn_crt_set (crt);
         }
       else
         {
-          get_country_crt_set (crt);
-          get_organization_crt_set (crt);
+          get_dn_crt_set (crt);
+
+          get_cn_crt_set (crt);
+          get_uid_crt_set (crt);
           get_unit_crt_set (crt);
+          get_organization_crt_set (crt);
           get_locality_crt_set (crt);
           get_state_crt_set (crt);
-          get_cn_crt_set (crt);
+          get_country_crt_set (crt);
           get_dc_set (TYPE_CRT, crt);
-          get_uid_crt_set (crt);
+
           get_oid_crt_set (crt);
           get_key_purpose_set (crt);
 
@@ -420,6 +404,7 @@ generate_certificate (gnutls_privkey_t * ret_key,
       get_dns_name_set (TYPE_CRT, crt);
       get_uri_set (TYPE_CRT, crt);
       get_ip_addr_set (TYPE_CRT, crt);
+      get_policy_set (crt);
 
       if (server != 0)
         {
@@ -443,9 +428,8 @@ generate_certificate (gnutls_privkey_t * ret_key,
 
           pk = gnutls_x509_crt_get_pk_algorithm (crt, NULL);
 
-          if (pk != GNUTLS_PK_DSA)
-            {                   /* DSA keys can only sign.
-                                 */
+          if (pk == GNUTLS_PK_RSA)
+            { /* DSA and ECDSA keys can only sign. */
               result = get_sign_status (server);
               if (result)
                 usage |= GNUTLS_KEY_DIGITAL_SIGNATURE;
@@ -906,6 +890,9 @@ cmd_parser (int argc, char **argv)
   if (HAVE_OPT(GENERATE_PRIVKEY) || HAVE_OPT(GENERATE_REQUEST) ||
       HAVE_OPT(KEY_INFO) || HAVE_OPT(PGP_KEY_INFO))
     privkey_op = 1;
+    
+  if (HAVE_OPT(HEX_NUMBERS))
+    full_format = GNUTLS_CRT_PRINT_FULL_NUMBERS;
 
   if (HAVE_OPT(OUTFILE))
     {
@@ -941,7 +928,7 @@ cmd_parser (int argc, char **argv)
     req_key_type = GNUTLS_PK_ECC;
   else
     req_key_type = GNUTLS_PK_RSA;
-
+    
   default_dig = GNUTLS_DIG_UNKNOWN;
   if (HAVE_OPT(HASH))
     {
@@ -991,6 +978,11 @@ cmd_parser (int argc, char **argv)
 
   memset (&cinfo, 0, sizeof (cinfo));
   
+  if (HAVE_OPT(VERBOSE))
+    cinfo.verbose = 1;
+    
+  cinfo.cprint = HAVE_OPT(CPRINT);
+  
   if (HAVE_OPT(LOAD_PRIVKEY))
     cinfo.privkey = OPT_ARG(LOAD_PRIVKEY);
 
@@ -1027,7 +1019,14 @@ cmd_parser (int argc, char **argv)
     cinfo.pkcs_cipher = OPT_ARG(PKCS_CIPHER);
 
   if (HAVE_OPT(PASSWORD))
-    cinfo.password = OPT_ARG(PASSWORD);
+    {
+      cinfo.password = OPT_ARG(PASSWORD);
+      if (HAVE_OPT(GENERATE_PRIVKEY) && cinfo.pkcs8 == 0)
+        {
+          fprintf(stderr, "Assuming PKCS #8 format...\n");
+          cinfo.pkcs8 = 1;
+        }
+    }
 
   if (HAVE_OPT(NULL_PASSWORD))
     {
@@ -1058,7 +1057,7 @@ cmd_parser (int argc, char **argv)
   else if (HAVE_OPT(CERTIFICATE_INFO))
     certificate_info (0, &cinfo);
   else if (HAVE_OPT(DH_INFO))
-    dh_info (&cinfo);
+    dh_info (infile, outfile, &cinfo);
   else if (HAVE_OPT(CERTIFICATE_PUBKEY))
     certificate_info (1, &cinfo);
   else if (HAVE_OPT(KEY_INFO))
@@ -1070,9 +1069,9 @@ cmd_parser (int argc, char **argv)
   else if (HAVE_OPT(P12_INFO))
     pkcs12_info (&cinfo);
   else if (HAVE_OPT(GENERATE_DH_PARAMS))
-    generate_prime (1, &cinfo);
+    generate_prime (outfile, 1, &cinfo);
   else if (HAVE_OPT(GET_DH_PARAMS))
-    generate_prime (0, &cinfo);
+    generate_prime (outfile, 0, &cinfo);
   else if (HAVE_OPT(CRL_INFO))
     crl_info ();
   else if (HAVE_OPT(P7_INFO))
@@ -1303,7 +1302,7 @@ pgp_privkey_info (void)
             fprintf (stderr, "Error in key RSA data export: %s\n",
                      gnutls_strerror (ret));
           else
-            print_rsa_pkey (outfile, &m, &e, &d, &p, &q, &u, NULL, NULL);
+            print_rsa_pkey (outfile, &m, &e, &d, &p, &q, &u, NULL, NULL, HAVE_OPT(CPRINT));
 
           bits = m.size * 8;
         }
@@ -1322,7 +1321,7 @@ pgp_privkey_info (void)
             fprintf (stderr, "Error in key DSA data export: %s\n",
                      gnutls_strerror (ret));
           else
-            print_dsa_pkey (outfile, &x, &y, &p, &q, &g);
+            print_dsa_pkey (outfile, &x, &y, &p, &q, &g, HAVE_OPT(CPRINT));
             
           bits = y.size * 8;
         }
@@ -1447,7 +1446,7 @@ print_certificate_info (gnutls_x509_crt_t crt, FILE * out, unsigned int all)
   int ret;
 
   if (all)
-    ret = gnutls_x509_crt_print (crt, GNUTLS_CRT_PRINT_FULL, &data);
+    ret = gnutls_x509_crt_print (crt, full_format, &data);
   else
     ret = gnutls_x509_crt_print (crt, GNUTLS_CRT_PRINT_UNSIGNED_FULL, &data);
   if (ret == 0)
@@ -1470,7 +1469,7 @@ print_crl_info (gnutls_x509_crl_t crl, FILE * out)
   int ret;
   size_t size;
 
-  ret = gnutls_x509_crl_print (crl, GNUTLS_CRT_PRINT_FULL, &data);
+  ret = gnutls_x509_crl_print (crl, full_format, &data);
   if (ret < 0)
     error (EXIT_FAILURE, 0, "crl_print: %s", gnutls_strerror (ret));
 
@@ -1525,7 +1524,7 @@ print_crq_info (gnutls_x509_crq_t crq, FILE * out)
 
   if (outcert_format == GNUTLS_X509_FMT_PEM)
     {
-      ret = gnutls_x509_crq_print (crq, GNUTLS_CRT_PRINT_FULL, &data);
+      ret = gnutls_x509_crq_print (crq, full_format, &data);
       if (ret < 0)
         error (EXIT_FAILURE, 0, "crq_print: %s", gnutls_strerror (ret));
 
@@ -1584,22 +1583,23 @@ crq_info (void)
 
 static void privkey_info_int (common_info_st* cinfo, gnutls_x509_privkey_t key)
 {
-int ret, key_type, bits = 0;
+int ret, key_type;
+unsigned int bits = 0;
 size_t size;
 const char *cprint;
 
   /* Public key algorithm
    */
   fprintf (outfile, "Public Key Info:\n");
-  ret = gnutls_x509_privkey_get_pk_algorithm (key);
+  ret = gnutls_x509_privkey_get_pk_algorithm2 (key, &bits);
   fprintf (outfile, "\tPublic Key Algorithm: ");
 
   key_type = ret;
 
   cprint = gnutls_pk_algorithm_get_name (key_type);
   fprintf (outfile, "%s\n", cprint ? cprint : "Unknown");
-  fprintf (outfile, "\tKey Security Level: %s\n\n",
-           gnutls_sec_param_get_name (gnutls_x509_privkey_sec_param (key)));
+  fprintf (outfile, "\tKey Security Level: %s (%u bits)\n\n",
+           gnutls_sec_param_get_name (gnutls_x509_privkey_sec_param (key)), bits);
 
   /* Print the raw public and private keys
    */
@@ -1615,8 +1615,7 @@ const char *cprint;
                  gnutls_strerror (ret));
       else
         {
-          print_rsa_pkey (outfile, &m, &e, &d, &p, &q, &u, &exp1, &exp2);
-          bits = m.size * 8;
+          print_rsa_pkey (outfile, &m, &e, &d, &p, &q, &u, &exp1, &exp2, HAVE_OPT(CPRINT));
 
           gnutls_free (m.data);
           gnutls_free (e.data);
@@ -1638,8 +1637,7 @@ const char *cprint;
                  gnutls_strerror (ret));
       else
         {
-          print_dsa_pkey (outfile, &x, &y, &p, &q, &g);
-          bits = y.size * 8;
+          print_dsa_pkey (outfile, &x, &y, &p, &q, &g, HAVE_OPT(CPRINT));
 
           gnutls_free (x.data);
           gnutls_free (y.data);
@@ -1659,8 +1657,7 @@ const char *cprint;
                  gnutls_strerror (ret));
       else
         {
-          print_ecc_pkey (outfile, curve, &k, &x, &y);
-          bits = gnutls_ecc_curve_get_size(curve) * 8;
+          print_ecc_pkey (outfile, curve, &k, &x, &y, HAVE_OPT(CPRINT));
 
           gnutls_free (x.data);
           gnutls_free (y.data);
@@ -1752,7 +1749,7 @@ generate_request (common_info_st * cinfo)
   gnutls_x509_privkey_t xkey;
   gnutls_pubkey_t pubkey;
   gnutls_privkey_t pkey;
-  int ret, ca_status, path_len;
+  int ret, ca_status, path_len, pk;
   const char *pass;
   unsigned int usage = 0;
 
@@ -1783,14 +1780,19 @@ generate_request (common_info_st * cinfo)
 
   pubkey = load_public_key_or_import (1, pkey, cinfo);
 
+  pk = gnutls_pubkey_get_pk_algorithm (pubkey, NULL);
+
   /* Set the DN.
    */
-  get_country_crq_set (crq);
-  get_organization_crq_set (crq);
+  get_dn_crq_set (crq);
+
+  get_cn_crq_set (crq);
   get_unit_crq_set (crq);
+  get_organization_crq_set (crq);
   get_locality_crq_set (crq);
   get_state_crq_set (crq);
-  get_cn_crq_set (crq);
+  get_country_crq_set (crq);
+
   get_dc_set (TYPE_CRQ, crq);
   get_uid_crq_set (crq);
   get_oid_crq_set (crq);
@@ -1822,14 +1824,21 @@ generate_request (common_info_st * cinfo)
         error (EXIT_FAILURE, 0, "set_basic_constraints: %s",
                gnutls_strerror (ret));
 
-      ret = get_sign_status (1);
-      if (ret)
-        usage |= GNUTLS_KEY_DIGITAL_SIGNATURE;
+      if (pk == GNUTLS_PK_RSA)
+        {
+          ret = get_sign_status (1);
+          if (ret)
+            usage |= GNUTLS_KEY_DIGITAL_SIGNATURE;
 
-      ret = get_encrypt_status (1);
-      if (ret)
-        usage |= GNUTLS_KEY_KEY_ENCIPHERMENT;
-      else
+          /* Only ask for an encryption certificate
+           * if it is an RSA one */
+          ret = get_encrypt_status (1);
+          if (ret)
+            usage |= GNUTLS_KEY_KEY_ENCIPHERMENT;
+          else
+            usage |= GNUTLS_KEY_DIGITAL_SIGNATURE;
+        }
+      else /* DSA and ECDSA are always signing */
         usage |= GNUTLS_KEY_DIGITAL_SIGNATURE;
 
       if (ca_status)
@@ -1987,7 +1996,7 @@ static int detailed_verification(gnutls_x509_crt_t cert,
   fprintf (outfile, "\tOutput: ");
   print_verification_res(outfile, verification_output);
 
-  fputs(".\n\n", outfile);
+  fputs("\n\n", outfile);
 
   return 0;
 }
@@ -2080,7 +2089,7 @@ _verify_x509_mem (const void *cert, int cert_size, const void* ca, int ca_size)
   fprintf (outfile, "Chain verification output: ");
   print_verification_res(outfile, output);
 
-  fprintf (outfile, ".\n\n");
+  fprintf (outfile, "\n\n");
 
   gnutls_free(x509_cert_list);
   gnutls_x509_trust_list_deinit(list, 1);
@@ -2094,58 +2103,27 @@ _verify_x509_mem (const void *cert, int cert_size, const void* ca, int ca_size)
 static void
 print_verification_res (FILE* outfile, unsigned int output)
 {
-  int comma = 0;
+  gnutls_datum_t pout;
+  int ret;
 
-  if (output & GNUTLS_CERT_INVALID)
+  if (output)
     {
-      fprintf (outfile, "Not verified");
-      comma = 1;
+      fprintf (outfile, "Not verified.");
     }
   else
     {
-      fprintf (outfile, "Verified");
-      comma = 1;
+      fprintf (outfile, "Verified.");
     }
 
-  if (output & GNUTLS_CERT_SIGNER_NOT_CA)
+  ret = gnutls_certificate_verification_status_print( output, GNUTLS_CRT_X509, &pout, 0);
+  if (ret < 0)
     {
-      if (comma)
-        fprintf (outfile, ", ");
-      fprintf (outfile, "Issuer is not a CA");
-      comma = 1;
+      fprintf(stderr, "error: %s\n", gnutls_strerror(ret));
+      exit(EXIT_FAILURE);
     }
 
-  if (output & GNUTLS_CERT_INSECURE_ALGORITHM)
-    {
-      if (comma)
-        fprintf (outfile, ", ");
-      fprintf (outfile, "Insecure algorithm");
-      comma = 1;
-    }
-
-  if (output & GNUTLS_CERT_NOT_ACTIVATED)
-    {
-      if (comma)
-        fprintf (outfile, ", ");
-      fprintf (outfile, "Not activated");
-      comma = 1;
-    }
-
-  if (output & GNUTLS_CERT_EXPIRED)
-    {
-      if (comma)
-        fprintf (outfile, ", ");
-      fprintf (outfile, "Expired");
-      comma = 1;
-    }
-
-  if (output & GNUTLS_CERT_REVOKED)
-    {
-      if (comma)
-        fprintf (outfile, ", ");
-      fprintf (outfile, "Revoked");
-      comma = 1;
-    }
+  fprintf (outfile, " %s", pout.data);
+  gnutls_free(pout.data);
 }
 
 static void
@@ -2199,11 +2177,9 @@ verify_crl (common_info_st * cinfo)
   size_t size, dn_size;
   char dn[128];
   unsigned int output;
-  int comma = 0;
   int ret;
-  gnutls_datum_t pem;
+  gnutls_datum_t pem, pout;
   gnutls_x509_crl_t crl;
-  time_t now = time (0);
   gnutls_x509_crt_t issuer;
 
   issuer = load_ca_cert (cinfo);
@@ -2236,51 +2212,24 @@ verify_crl (common_info_st * cinfo)
   if (ret < 0)
     error (EXIT_FAILURE, 0, "verification error: %s", gnutls_strerror (ret));
 
-  if (output & GNUTLS_CERT_INVALID)
+  if (output)
     {
-      fprintf (outfile, "Not verified");
-      comma = 1;
+      fprintf (outfile, "Not verified. ");
     }
   else
     {
-      fprintf (outfile, "Verified");
-      comma = 1;
+      fprintf (outfile, "Verified.");
     }
 
-  if (output & GNUTLS_CERT_SIGNER_NOT_CA)
+  ret = gnutls_certificate_verification_status_print( output, GNUTLS_CRT_X509, &pout, 0);
+  if (ret < 0)
     {
-      if (comma)
-        fprintf (outfile, ", ");
-      fprintf (outfile, "Issuer is not a CA");
-      comma = 1;
+      fprintf(stderr, "error: %s\n", gnutls_strerror(ret));
+      exit(EXIT_FAILURE);
     }
 
-  if (output & GNUTLS_CERT_INSECURE_ALGORITHM)
-    {
-      if (comma)
-        fprintf (outfile, ", ");
-      fprintf (outfile, "Insecure algorithm");
-      comma = 1;
-    }
-
-  /* Check expiration dates.
-   */
-
-  if (gnutls_x509_crl_get_this_update (crl) > now)
-    {
-      if (comma)
-        fprintf (outfile, ", ");
-      comma = 1;
-      fprintf (outfile, "Issued in the future!");
-    }
-
-  if (gnutls_x509_crl_get_next_update (crl) < now)
-    {
-      if (comma)
-        fprintf (outfile, ", ");
-      comma = 1;
-      fprintf (outfile, "CRL is not up to date");
-    }
+  fprintf (outfile, " %s", pout.data);
+  gnutls_free(pout.data);
 
   fprintf (outfile, "\n");
 }
@@ -2770,6 +2719,8 @@ void
 pubkey_info (gnutls_x509_crt_t crt, common_info_st * cinfo)
 {
   gnutls_pubkey_t pubkey;
+  gnutls_privkey_t privkey = NULL;
+  gnutls_x509_crq_t crq = NULL;
   int ret;
   size_t size;
 
@@ -2784,18 +2735,41 @@ pubkey_info (gnutls_x509_crt_t crt, common_info_st * cinfo)
       crt = load_cert (0, cinfo);
     }
 
+  if (crq == NULL)
+    {
+      crq = load_request (cinfo);
+    }
+
   if (crt != NULL)
     {
       ret = gnutls_pubkey_import_x509 (pubkey, crt, 0);
       if (ret < 0)
-        {
-          error (EXIT_FAILURE, 0, "pubkey_import_x509: %s",
-                 gnutls_strerror (ret));
-        }
+        error (EXIT_FAILURE, 0, "pubkey_import_x509: %s",
+               gnutls_strerror (ret));
+    }
+  else if (crq != NULL)
+    {
+      ret = gnutls_pubkey_import_x509_crq (pubkey, crq, 0);
+      if (ret < 0)
+        error (EXIT_FAILURE, 0, "pubkey_import_x509_crq: %s",
+               gnutls_strerror (ret));
     }
   else
     {
-      pubkey = load_pubkey (1, cinfo);
+      privkey = load_private_key (0, cinfo);
+      
+      if (privkey != NULL)
+        {
+          ret = gnutls_pubkey_import_privkey(pubkey, privkey, 0, 0);
+          if (ret < 0)
+            error (EXIT_FAILURE, 0, "pubkey_import_privkey: %s",
+                   gnutls_strerror (ret));
+        }
+      else
+        {
+          gnutls_pubkey_deinit(pubkey);
+          pubkey = load_pubkey (1, cinfo);
+        }
     }
 
   if (outcert_format == GNUTLS_X509_FMT_DER)
@@ -2814,6 +2788,6 @@ pubkey_info (gnutls_x509_crt_t crt, common_info_st * cinfo)
     
   /* PEM */
 
-  _pubkey_info(outfile, pubkey);
+  _pubkey_info(outfile, full_format, pubkey);
   gnutls_pubkey_deinit (pubkey);
 }
