@@ -302,6 +302,7 @@ get_params (gnutls_session_t session, gnutls_params_type_t type,
   return 0;
 }
 
+#ifdef ENABLE_RSA_EXPORT
 static int
 generate_rsa_params (void)
 {
@@ -327,6 +328,13 @@ generate_rsa_params (void)
 
   return 0;
 }
+#else
+static int
+generate_rsa_params (void) 
+{
+  return 0;
+}
+#endif
 
 LIST_DECLARE_INIT (listener_list, listener_item, listener_free);
 
@@ -355,11 +363,6 @@ gnutls_session_t initialize_session (int dtls)
       gnutls_db_set_store_function (session, wrap_db_store);
       gnutls_db_set_ptr (session, NULL);
     }
-#ifdef ENABLE_SESSION_TICKET
-  if (noticket == 0)
-    gnutls_session_ticket_enable_server (session, &session_ticket_key);
-#endif
-
 
   if (noticket == 0)
     gnutls_session_ticket_enable_server (session, &session_ticket_key);
@@ -943,6 +946,7 @@ main (int argc, char **argv)
 {
   int ret, mtu, port;
   char name[256];
+  int cert_set = 0;
 
   set_program_name (argv[0]);
   cmd_parser (argc, argv);
@@ -1053,7 +1057,7 @@ main (int argc, char **argv)
         }
     }
 
-  if (HAVE_OPT (PGPCERTFILE))
+  if (pgp_certfile != NULL && pgp_keyfile != NULL)
     {
       if (HAVE_OPT (PGPSUBKEY))
         ret = gnutls_certificate_set_openpgp_key_file2
@@ -1070,38 +1074,60 @@ main (int argc, char **argv)
                    ret, pgp_certfile, pgp_keyfile);
           GERR (ret);
         }
+      else
+        cert_set = 1;
     }
 #endif
 
-  if (x509_certfile != NULL)
-    if ((ret = gnutls_certificate_set_x509_key_file
-         (cert_cred, x509_certfile, x509_keyfile, x509ctype)) < 0)
-      {
-        fprintf (stderr,
+  if (x509_certfile != NULL && x509_keyfile != NULL)
+    {
+      ret = gnutls_certificate_set_x509_key_file
+         (cert_cred, x509_certfile, x509_keyfile, x509ctype);
+      if (ret < 0)
+        {
+          fprintf (stderr,
                  "Error reading '%s' or '%s'\n", x509_certfile, x509_keyfile);
-        GERR (ret);
-        exit (1);
-      }
+          GERR (ret);
+          exit (1);
+        }
+      else
+        cert_set = 1;
+    }
 
-  if (x509_dsacertfile != NULL)
-    if ((ret = gnutls_certificate_set_x509_key_file
-         (cert_cred, x509_dsacertfile, x509_dsakeyfile, x509ctype)) < 0)
-      {
-        fprintf (stderr, "Error reading '%s' or '%s'\n",
-                 x509_dsacertfile, x509_dsakeyfile);
-        GERR (ret);
-        exit (1);
-      }
+  if (x509_dsacertfile != NULL && x509_dsakeyfile != NULL)
+    {
+      ret = gnutls_certificate_set_x509_key_file
+         (cert_cred, x509_dsacertfile, x509_dsakeyfile, x509ctype);
+      if (ret < 0)
+        {
+          fprintf (stderr,
+                 "Error reading '%s' or '%s'\n", x509_dsacertfile, x509_dsakeyfile);
+          GERR (ret);
+          exit (1);
+        }
+      else
+        cert_set = 1;
+    }
 
-  if (x509_ecccertfile != NULL)
-    if ((ret = gnutls_certificate_set_x509_key_file
-         (cert_cred, x509_ecccertfile, x509_ecckeyfile, x509ctype)) < 0)
-      {
-        fprintf (stderr, "Error reading '%s' or '%s'\n",
-                 x509_ecccertfile, x509_ecckeyfile);
-        GERR (ret);
-        exit (1);
-      }
+  if (x509_ecccertfile != NULL && x509_ecckeyfile != NULL)
+    {
+      ret = gnutls_certificate_set_x509_key_file
+         (cert_cred, x509_ecccertfile, x509_ecckeyfile, x509ctype);
+      if (ret < 0)
+        {
+          fprintf (stderr,
+                 "Error reading '%s' or '%s'\n", x509_ecccertfile, x509_ecckeyfile);
+          GERR (ret);
+          exit (1);
+        }
+      else
+        cert_set = 1;
+    }
+    
+  if (cert_set == 0) 
+    {
+      fprintf(stderr, "Warning: no private key and certificate pairs were set.\n");
+    }
 
   /* OCSP status-request TLS extension */
   if (status_response_ocsp)
@@ -1296,9 +1322,7 @@ tcp_server (const char *name, int port)
                 j->fd = accept_fd;
 
                 j->tls_session = tls_session;
-                gnutls_transport_set_ptr (tls_session,
-                                          (gnutls_transport_ptr_t)
-                                          gl_fd_to_handle (accept_fd));
+                gnutls_transport_set_int (tls_session, accept_fd);
                 j->handshake_ok = 0;
 
                 if (verbose != 0)
